//
//  Entry.h
//  iOrdning
//
//  Created by Erik Aderstedt on 2007-10-30.
//  Copyright 2007 Aderstedt Software AB. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class Book;
@class FiscalYear;

#define NEW_NO	2147483647
#define SIE_MISSING_NUMBER_NO	2147383647

#define AS_ET_UNKNOWN
#define AS_ET_RECEIVED_SUPPLIER_INVOICE 1
#define AS_ET_PAID_SUPPLIER_INVOICE 2

enum ASEntryType {
    kASEntryTypeUnknown = 0,
    kASEntryTypeSent,
    kASEntryTypePartiallyPaid,
    kASEntryTypePaid,
    kASEntryTypeReminder,
    kASEntryTypeLoss
};

@interface Entry : NSManagedObject <NSCopying>{

}

@property(retain) NSString	*name;
@property(retain) NSNumber	*number;
@property(retain) NSString	*comments;
@property(retain) NSImage	*image;
@property(retain) NSNumber	*date;
@property(retain) NSNumber	*type;
@property(retain) FiscalYear *fiscalYear;
@property(retain) NSSet		*rows;


- (void)setInvoiceEntryType:(enum ASEntryType)entryType;

+ (NSPredicate *)onlyNewEntries;
+ (Entry *)findTheNewInContext:(NSManagedObjectContext *)moc;

+ (NSData *)reduceImageWithData:(NSData *)data;

- (NSString *)entryDescription;
- (NSDictionary *)contents;
- (NSNumber *)isNew;
- (NSString *)numberAsString;
- (NSString *)dateString;
- (NSArray *)orderedRows;
- (NSDecimalNumber *)turnover; /* Omslutning */
- (NSNumber *)nextEntryNumber;
- (BOOL)isBalanced;

- (void)copyRowsFromOtherEntry:(Entry *)otherEntry;
- (void)defineRowsFromXml:(NSXMLElement *)xmlEntry;
- (void)removeEmptyRows;
+ (void)removeBadRowsInNewEntryInContext:(NSManagedObjectContext *)moc;

- (BOOL)checkValidation:(NSError **)error;
- (BOOL)editFinished:(NSError **)error;
- (NSArray *)fileMe:(NSError **)error;
- (NSArray *)fileMe:(NSError **)error toUpcoming:(BOOL)toUpcoming;
- (void)clear;

- (NSString *)matchingAmounts;
- (NSString *)matchingTags;
- (NSString *)textualRepresentation;
- (BOOL)hasTrackedChanges;

- (BOOL)validateRows:(id *)ioValues error:(NSError **)outError;
- (BOOL)validateDate:(id *)n error:(NSError **)e;
@end
