//
//  Invoice.h
//  iOrdning
//
//  Created by Erik Aderstedt on 2008-04-28.
//  Copyright 2008 Aderstedt Software AB. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#define UNKNOWN 0
#define SENT 1
#define OVERDUE 2
#define PAID 3
#define REMINDER 4
#define REMINDER2 5
#define LOSS 6
#define PARTIALLY_PAID 7
#define RETURNED 8

enum InvoiceStatus {
	AS_IS_NOTSENT='Nose',
	AS_IS_SENT='Sent',
	AS_IS_OVERDUE='Over',
	AS_IS_PAID='Paid',
	AS_IS_REMINDER='Rem1',
	AS_IS_REMINDER2='Rem2',
	AS_IS_LOSS='Loss',
	AS_IS_PARTIALLY_PAID='PPai',
	AS_IS_RETURNED='Retu'
};

enum ASInvoiceStatus {
    kASInvoiceStatusUnknown,
    kASInvoiceStatusSent,
    kASInvoiceStatusOverdue,
    kASInvoiceStatusPaid,
    kASInvoiceStatusReminder,
    kASInvoiceStatusReminder2,
    kASInvoiceStatusLoss,
    kASInvoiceStatusPartiallyPaid,
    kASInvoiceStatusReturned
};

@class Book;
@class Entry;
@class Allowance;
@class Tag;
@class Account;
@class Article;
@class Language;
@class Template;

//
// Sending an invoice:
//
// -(BOOL)sendWithExchangeRate:(NSNumber *)eRate error:(NSError **)error
//
// Attempts to send an invoice with the given exchange rate.
//

@interface Invoice : NSManagedObject {
	NSNumber *vatOK;
}

@property(retain) NSNumber *vatOK;
@property(retain) NSString *attention;
@property(retain) Book *book;
@property(retain) NSNumber *bookOnce;
@property(retain) NSString *city;
@property(retain) NSString *company;
@property(retain) NSString *county;
@property(retain) NSString *country;
@property(retain) NSString *creditDetails;
@property(retain) NSString *currency;
@property(retain) NSNumber *custLossDate;
@property(retain) NSNumber *date;
@property(retain) NSNumber *days;
@property(retain) NSString *deliveryMethod;
@property(retain) NSNumber *dueDate;
@property(retain) NSDecimalNumber *exchangeRate;
@property(retain) NSString *language;
@property(retain) NSString *note;
@property(retain) NSNumber *number;
@property(retain) NSString *ourRef;
@property(retain) NSDecimalNumber *localAmount;
@property(retain) NSDecimalNumber *paidAmount;
@property(retain) NSNumber *paidDate;
@property(retain) NSString *purchaseOrderNo;
@property(retain) NSNumber *rounding;
@property(retain) NSNumber *status;
@property(retain) NSString *street;
@property(retain) NSNumber *vatfree;
@property(retain) NSString *vatno;
@property(retain) NSString *yourRef;
@property(retain) NSString *zipcode;
@property(retain) NSSet *lines;
@property(retain) Allowance *allowance;
@property(retain) NSString *allowanceString;
@property(retain) Tag *tag;
@property(retain) NSString *templateName;
- (void)setPropertiesFromLatestInvoice;

// Dependent keys.
- (NSNumber *)sendable;
- (NSDecimalNumber *)totalAmount;
- (NSDecimalNumber *)totalAmountExclVat;
- (NSDecimalNumber *)vatAmount;
- (NSDecimalNumber *)roundingAmount;
- (NSDecimalNumber *)rebateAmount;
- (NSDecimalNumber *)amountRemaining;
- (NSString *)recipient;
@property(nonatomic,retain) Template *invoiceTemplate;

- (NSDictionary *)amountsForEachVatLevel;
- (NSString *)vatSummary;
- (NSString *)vatSummaryUsingFormatter:(NSNumberFormatter *)formatter;
- (NSNumber *)isEmpty;
- (NSNumber *)isPaid;
- (NSNumber *)isOverdue;
- (NSNumber *)canBePaid;
- (NSNumber *)isSent ;
- (NSNumber *)isLossOrReturn;
- (NSString *)creditDetails;
- (NSNumber *)days;
- (NSAttributedString *)indicator;
- (NSNumber *)status;
- (NSNumber *)rawStatus;
- (void)setRawStatus:(NSNumber *)s;
- (NSArray *)sortedInvoiceLines;
- (NSNumber *)foreignCurrency;
- (NSNumber *)hasDeliveryAddress;
- (void)setHasDeliveryAddress:(NSNumber *)n;
- (NSString *)shownDeliveryAddress;

- (NSString *)titleForLanguage:(Language *)l;
- (NSString *)reminderForLanguage:(Language *)l;

// State actions
- (void)remindWithArticle:(Article *)reminderArticle;
- (BOOL)assignCustomer:(NSError **)error;
- (BOOL)send:(NSError **)error;
- (BOOL)payMeWithFinalAmount:(NSDecimalNumber *)finalAmount onDateIndex:(NSNumber *)paymentDay inFull:(BOOL)full receivingAccount:(Account *)receivingAccount error:(NSError **)error;
- (BOOL)registerReturnWithError:(NSError **)error usingAccount:(Account *)a;
- (BOOL)registerCustomerLossWithError:(NSError **)error usingAccount:(Account *)a;

// Helper functions
- (Entry *)insertReversedSalesEntry:(NSError **)error withAccountForAlreadyPaid:(Account *)a;

+ (NSString *)sendEntryNameForNumber:(NSObject *)number;
- (NSString *)sendEntryName;
+ (NSString *)paidEntryNameForNumber:(NSObject *)number;
- (NSString *)paidEntryName;
+ (NSString *)partiallyPaidEntryNameForNumber:(NSObject *)number;
- (NSString *)partiallyPaidName;

- (BOOL)addSaleRowsToEntry:(Entry *)entry withExchangeRate:(NSDecimalNumber *)eRate;
- (void)updateTotalAmount;
- (NSManagedObject *)createDeliveryAddress;
- (void)removeEmptyLines;

- (Invoice *)duplicate;

+ (NSMutableAttributedString *)attributedStringIndicatorForInvoiceStatus:(NSNumber *)status;

- (NSNumber *)canRevertToNotSent;
- (void)revertToNotSent;

@end
